
# q-expansion of newform 400.2.u.d, downloaded from the LMFDB on 14 May 2024.

# We generate the q-expansion using the Hecke eigenvalues a_p at the primes.
# Each a_p is given as a linear combination
# of the following basis for the coefficient ring.

def make_data():

    from sage.all import prod, floor, prime_powers, gcd, QQ, primes_first_n, next_prime, RR

    def discrete_log(elts, gens, mod):
        # algorithm 2.2, page 16 of https://arxiv.org/abs/0903.2785
        def table_gens(gens, mod):
            T = [1]
            n = len(gens)
            r = [None]*n
            s = [None]*n
            for i in range(n):
                beta = gens[i]
                r[i] = 1
                N = len(T)
                while beta not in T:
                    for Tj in T[:N]:
                        T.append((beta*Tj) % mod)
                    beta = (beta*gens[i]) % mod
                    r[i] += 1
                s[i] = T.index(beta)
            return T, r, s
        T, r, s = table_gens(gens, mod)
        n = len(gens)
        N = [ prod(r[:j]) for j in range(n) ]
        Z = lambda s: [ (floor(s/N[j]) % r[j]) for j in range(n)]
        return [Z(T.index(elt % mod)) for elt in elts]
    def extend_multiplicatively(an):
        for pp in prime_powers(len(an)-1):
            for k in range(1, (len(an) - 1)//pp + 1):
                if gcd(k, pp) == 1:
                    an[pp*k] = an[pp]*an[k]
    from sage.all import PolynomialRing, NumberField, ZZ
    R = PolynomialRing(QQ, "x")
    f = R(poly_data)
    K = NumberField(f, "a")
    betas = [K([c/ZZ(den) for c in num]) for num, den in basis_data]
    convert_elt_to_field = lambda elt: sum(c*beta for c, beta in zip(elt, betas))
    # convert aps to K elements
    primes = primes_first_n(len(aps_data))
    good_primes = [p for p in primes if not p.divides(level)]
    aps = map(convert_elt_to_field, aps_data)
    if not hecke_ring_character_values:
        # trivial character
        char_values = dict(zip(good_primes, [1]*len(good_primes)))
    else:
        gens = [elt[0] for elt in hecke_ring_character_values]
        gens_values = [convert_elt_to_field(elt[1]) for elt in hecke_ring_character_values]
        char_values = dict([(
            p,prod(g**k for g, k in zip(gens_values, elt)))
            for p, elt in zip(good_primes, discrete_log(good_primes, gens, level))
            ])
    an_list_bound = next_prime(primes[-1])
    an = [0]*an_list_bound
    an[1] = 1
    
    from sage.all import PowerSeriesRing
    PS = PowerSeriesRing(K, "q")
    for p, ap in zip(primes, aps):
        if p.divides(level):
            euler_factor = [1, -ap]
        else:
            euler_factor = [1, -ap, p**(weight - 1) * char_values[p]]
        k = RR(an_list_bound).log(p).floor() + 1
        foo = (1/PS(euler_factor)).padded_list(k)
        for i in range(1, k):
            an[p**i] = foo[i]
    extend_multiplicatively(an)
    return PS(an)
level = 400
weight = 2
poly_data = [25, -25, -10, 5, 11, -7, 4, -3, 1]

# The entries in the following list give a basis for the
# coefficient ring in terms of a root of the defining polynomial above.
# Each line consists of the coefficients of the numerator, and a denominator.
basis_data  = [[[1, 0, 0, 0, 0, 0, 0, 0], 1], [[235, 2175, -126, -732, 399, -318, 206, -64], 1355], [[-7800, 1065, 4949, 2103, -1896, 747, -714, 406], 1355], [[10265, -1055, -5129, -2297, 1924, -698, 776, -420], 1355], [[-11505, 3705, 5125, 2919, -2363, 946, -877, 504], 1355], [[12985, -530, -6178, -4352, 3394, -1513, 1321, -613], 1355], [[14605, -2200, -8367, -3390, 3325, -1295, 1265, -714], 1355], [[21445, -3530, -10443, -6314, 5563, -2121, 1962, -1020], 1355]]

hecke_ring_character_values = [[351, [1, 0, 0, 0, 0, 0, 0, 0]], [101, [1, 0, 0, 0, 0, 0, 0, 0]], [177, [0, 0, 0, -1, 0, 0, 0, 0]]]
aps_data = [[0, 0, 0, 0, 0, 0, 0, 0], [0, 0, 1, 1, 0, 1, 1, 0], [0, 1, 0, 1, 0, 0, 0, 0], [-1, 1, 0, 0, -1, -1, 0, 0], [0, 1, 0, 0, 0, -1, -1, 0], [-3, -1, 3, 1, 1, 0, 1, -1], [-1, 0, -2, 0, 0, 1, 1, -1], [-3, 0, 1, 0, 1, 1, 3, -1], [2, -1, 0, -2, 2, 1, -3, -2], [0, 0, -2, -5, 0, -1, -2, 0], [2, 0, -5, 0, -2, -1, -2, 1], [-3, 0, 3, 2, 0, 0, 2, -2], [-3, -1, 3, 3, 1, 0, 3, 1], [-3, -1, 4, 4, 1, 1, 0, 0], [0, 1, 0, 1, 0, -1, 0, 1], [0, 0, 4, 0, 0, 3, 4, 0], [-2, 2, 2, 2, -2, 0, 2, 4], [1, 4, 0, -1, -2, -4, -3, 2], [0, 0, 7, 0, 0, 3, 0, -3], [0, 1, 4, -3, 0, -1, 4, 1], [2, -1, 0, -2, 3, 1, -8, -3], [0, 0, 2, -2, 0, -2, 2, 0], [-2, 0, -5, 0, 2, -1, 2, 1], [-4, 4, 0, 4, -1, -4, 2, 1], [0, 0, -5, -8, 0, 0, -5, 0], [3, -6, -5, -5, 0, 0, 0, 0], [0, -2, -1, 1, 0, 1, -1, -2], [5, -2, -9, -9, -3, -3, 0, 0], [-7, 3, 7, 5, -3, 0, 5, 3], [5, 0, -5, 1, 0, 0, 1, 1], [0, 2, 0, 0, 0, -2, 2, 0], [-6, 0, 1, 0, 2, 1, 6, -1], [3, -3, -3, -10, 3, 0, -10, -2], [-5, -3, 0, 5, 3, 3, -7, -3], [-7, 4, 4, 4, -3, -3, 0, 0], [12, -2, 2, 2, 4, 4, 0, 0], [8, -1, 1, 1, 1, 1, 0, 0], [-3, 3, 3, -6, -3, 0, -6, -1], [-5, 0, 7, 0, 3, -1, 5, 1], [-2, 2, 0, 2, -3, -2, 12, 3], [0, 5, 0, 1, 0, -1, 0, 5], [-8, 0, -2, 0, -3, 1, 8, -1], [-7, 3, 7, 13, -3, 0, 13, 2], [-8, 3, 14, 14, 4, 4, 0, 0], [0, 0, 4, 0, 0, -1, 4, 0], [-4, 3, -5, -5, -8, -8, 0, 0], [-9, 5, 0, 9, -5, -5, 9, 5], [-4, -3, 0, 4, 4, 3, 7, -4], [-10, -1, 0, 10, -2, 1, 5, 2], [0, -6, 5, 7, 0, 6, 5, -6], [6, 0, -4, 0, -3, 0, -6, 0], [-1, 0, 0, 1, -1, 0, 10, 1], [10, -5, -10, -1, 5, 0, -1, 0], [5, -1, -4, -4, -3, -3, 0, 0], [-9, 1, 3, 3, -1, -1, 0, 0], [-9, 1, 9, 16, -1, 0, 16, -1], [-3, 0, -12, 0, 4, -4, 3, 4], [0, -3, -11, -10, 0, 8, -11, -3], [-10, 2, 0, 10, 1, -2, 4, -1], [10, 0, -7, 0, 5, 5, -10, -5], [10, 0, -10, 0, -2, -2, -10, 2], [-20, -1, 13, 13, -3, -3, 0, 0], [-10, -4, 14, 14, -2, -2, 0, 0], [15, -5, 0, -15, 3, 5, -9, -3], [-14, 0, 14, 13, 0, 0, 13, 3], [-3, 0, -8, 0, 1, -6, 3, 6], [9, 0, 7, 0, 5, -1, -9, 1], [0, -2, 0, -4, 2, 0, -4, 4], [0, -1, -3, -20, 0, 2, -3, -1], [9, 5, -12, -12, -6, -6, 0, 0], [0, 0, 6, -4, 0, -2, 6, 0], [-12, -4, 12, 5, 4, 0, 5, -3], [-10, 0, 15, 0, 2, 3, 10, -3], [12, 5, 0, -12, 0, -5, -15, 0], [0, -11, 2, -5, 0, 9, 2, -11], [3, 0, 12, 0, -5, 4, -3, -4], [-3, -5, 0, 3, -6, 5, 6, 6], [0, -1, -4, -11, 0, 9, -4, -1], [22, 5, -2, -2, 2, 2, 0, 0], [-15, -4, 15, -7, 4, 0, -7, -11], [10, 0, -22, 0, -6, -6, -10, 6], [0, -2, 7, 14, 0, 6, 7, -2], [-2, 0, -12, 0, 2, 4, 2, -4], [-16, 0, -3, 0, -5, 1, 16, -1], [-10, -1, 0, 10, 2, 1, -11, -2], [-4, 12, 4, 4, -8, -8, 0, 0], [1, -7, 12, 12, 10, 10, 0, 0], [23, 2, -5, -5, 1, 1, 0, 0], [-2, -6, 0, 2, 9, 6, 4, -9], [3, -7, -3, -6, 7, 0, -6, -3], [20, 0, -13, 0, 0, 3, -20, -3], [0, -5, 11, -2, 0, 0, 11, -5], [-7, 3, 7, -7, -3, 0, -7, 6], [4, 0, -4, 2, 0, 0, 2, -6], [9, -8, -11, -11, 9, 9, 0, 0], [0, -3, -9, 4, 0, 2, -9, -3], [4, 1, -4, 13, -1, 0, 13, -4], [0, -1, -11, -20, 0, -7, -11, -1], [2, 8, 0, -2, 2, -8, 0, -2], [-27, 8, 27, 23, -8, 0, 23, 11], [0, 0, 7, 11, 0, 3, 7, 0], [-3, -5, 14, 14, -2, -2, 0, 0], [-5, 5, 5, 8, -5, 0, 8, 7], [2, 0, 11, 0, -9, 6, -2, -6], [0, -4, 7, -17, 0, 7, 7, -4], [16, -9, 0, -16, 12, 9, -13, -12], [18, -2, -18, -16, 2, 0, -16, -2], [-14, 10, 2, 2, -1, -1, 0, 0], [4, 8, 20, 20, -4, -4, 0, 0], [16, 1, -4, -4, 4, 4, 0, 0], [13, 14, -1, -1, 1, 1, 0, 0], [22, -5, -22, -17, 5, 0, -17, -4], [-8, 0, 6, 0, -9, 2, 8, -2], [12, 0, -11, 0, 0, 5, -12, -5], [16, 0, -2, 0, 12, 2, -16, -2], [-6, 4, 6, -2, -4, 0, -2, -8], [-31, -1, 0, 0, 1, 1, 0, 0], [0, -1, 2, -3, 0, -5, 2, -1], [0, 2, 0, 3, 0, -13, 0, 2], [-21, -3, 21, -3, 3, 0, -3, -8], [11, -16, 0, -11, 11, 16, -24, -11], [2, 0, 0, -2, 5, 0, 10, -5], [-1, -1, 0, 1, 11, 1, -19, -11], [-10, 0, -5, 0, -10, 7, 10, -7], [10, 6, -10, 21, -6, 0, 21, 3], [-10, 1, -2, -2, -6, -6, 0, 0], [-3, -9, 3, 11, 9, 0, 11, 1], [-7, 0, -4, 0, 5, 0, 7, 0], [14, 6, 0, -14, 2, -6, -6, -2], [16, 0, -16, 0, 8, -2, -16, 2], [8, -10, 0, -8, 12, 10, 2, -12], [-7, 4, 1, 1, -3, -3, 0, 0], [-11, 4, 1, 1, -11, -11, 0, 0], [5, 10, -10, -10, 1, 1, 0, 0], [-3, 2, 0, 3, -2, -2, -5, 2], [-4, 0, 0, 0, -12, 2, 4, -2], [25, -9, 0, -25, 6, 9, -14, -6], [24, -4, -24, -28, 4, 0, -28, -4], [0, -10, 15, 5, 0, 12, 15, -10], [-1, -10, 1, -17, 10, 0, -17, -7], [0, -8, 0, 0, -4, 8, 24, 4], [0, -3, -13, -10, 0, -2, -13, -3], [-4, -2, 0, 4, 0, 2, 2, 0], [-27, 5, 0, 27, -3, -5, 25, 3], [0, -2, -4, -10, 0, 9, -4, -2], [20, 1, 0, -20, 0, -1, -33, 0], [0, 11, -17, -14, 0, -9, -17, 11], [-14, 7, 7, 7, -12, -12, 0, 0], [5, -1, -5, -8, 1, 0, -8, 1], [-16, -8, 16, 22, 8, 0, 22, -2], [-3, 5, 0, 3, -10, -5, 16, 10], [22, 0, 2, 0, -2, -4, -22, 4], [-7, 0, 20, 0, 17, 4, 7, -4], [1, -1, -1, 25, 1, 0, 25, 2], [2, -3, 7, 7, 14, 14, 0, 0], [-12, 1, 0, 12, -4, -1, -17, 4], [-22, 0, 25, 0, -10, -7, 22, 7], [0, 2, 16, 3, 0, -1, 16, 2], [-27, 16, 27, 24, -16, 0, 24, 8], [-5, 2, 5, -8, -2, 0, -8, 2], [0, 11, 3, -18, 0, -8, 3, 11], [0, -4, 15, 4, 0, 4, 15, -4], [2, 0, 12, 0, 10, 20, -2, -20], [0, -4, -14, 10, 0, 2, -14, -4], [-18, -8, 0, 18, 11, 8, 3, -11], [6, 0, 13, 0, 2, -7, -6, 7], [5, -13, -5, -24, 13, 0, -24, 1], [0, 7, 30, 31, 0, -5, 30, 7]]
